User-agent: *
Disallow: /web/
Disallow: /en_GB/web/
Disallow: /pl_PL/web/
Disallow: /de_DE/web/
Disallow: /it_IT/web/
Disallow: /fr_FR/web/
Disallow: /ed_ES/web/
Disallow: /en_US/web/
Disallow: /ru_RU/web/
Disallow: /ua_UA/web/
Disallow: /zh_CN/web/
Disallow: /sk_SK/web/
Disallow: /group/
Disallow: /en_GB/group/
Disallow: /pl_PL/group/
Disallow: /de_DE/group/
Disallow: /it_IT/group/
Disallow: /fr_FR/group/
Disallow: /ed_ES/group/
Disallow: /en_US/group/
Disallow: /ru_RU/group/
Disallow: /ua_UA/group/
Disallow: /zh_CN/group/
Disallow: /sk_SK/group/
Disallow: /web/interdyscyplinarne-rejestracja/
Disallow: /group/interdyscyplinarne-rejestracja/
Disallow: /pipermail/
Disallow: /BIP/
Disallow: /documents/
Disallow: /c/document_library/
Disallow: /konkursy2012/

User-agent: googlebot
Disallow: /web/
Disallow: /en_GB/web/
Disallow: /pl_PL/web/
Disallow: /de_DE/web/
Disallow: /it_IT/web/
Disallow: /fr_FR/web/
Disallow: /ed_ES/web/
Disallow: /en_US/web/
Disallow: /ru_RU/web/
Disallow: /ua_UA/web/
Disallow: /zh_CN/web/
Disallow: /sk_SK/web/
Disallow: /group/
Disallow: /en_GB/group/
Disallow: /pl_PL/group/
Disallow: /de_DE/group/
Disallow: /it_IT/group/
Disallow: /fr_FR/group/
Disallow: /ed_ES/group/
Disallow: /en_US/group/
Disallow: /ru_RU/group/
Disallow: /ua_UA/group/
Disallow: /zh_CN/group/
Disallow: /sk_SK/group/
Disallow: /web/interdyscyplinarne-rejestracja/
Disallow: /group/interdyscyplinarne-rejestracja/
Disallow: /konkursy2012/