User-agent: *
Disallow: /buffer/
Disallow: /cumulus/
Disallow: /mirage/
Disallow: /eoffice/
Disallow: /eoffice2/
Disallow: /tinymce/
Disallow: /WEB-INF/
Disallow: /rss_list
Disallow: /rss_all
Disallow: /rss.xml
Disallow: /rss.dhtml
Disallow: /rss_1.xml
Disallow: /rss_2.xml
Disallow: /rss_3.xml
Disallow: /rss_4.xml
Disallow: /rss_5.xml
Disallow: /rss_1.dhtml
Disallow: /rss_2.dhtml
Disallow: /rss_3.dhtml
Disallow: /rss_4.dhtml
Disallow: /rss_5.dhtml
Disallow: /rss-1
Disallow: /rss-2
Disallow: /rss-3
Disallow: /rss-4
Disallow: /rss-5
Disallow: /rss
Disallow: /undefined/twitter_proxy.xml
Disallow: /twitter_proxy.xml
Disallow: /ajax_ads.xml
Sitemap: http://www.wroclaw.pl/sitemap.xml?alias=index
